﻿using System;
using System.IO;
using System.Linq;
using Dss.Api.Examples.Internal;
using Dss.Api.Examples.Properties;
using DataScope.Select.Api.Content;
using DataScope.Select.Api.Extractions;
using DataScope.Select.Api.Extractions.ExtractionRequests;
using DataScope.Select.Api.Extractions.ReportExtractions;
using DataScope.Select.Api.Extractions.ReportTemplates;
using DataScope.Select.Api.Extractions.Schedules;
using DataScope.Select.Api.Extractions.SubjectLists;

namespace Dss.Api.Examples.QuickStart
{
    [Example("1. Quick Start")]
    public class QuickStartExamples
    {
        //DO NOT CHANGE THE EXAMPLE NAME, the name is used verbatim in the Help documentation site.
        [Example("Authenticate: Primary Example")]
        [EndToEndExample]
        public void Authentication()
        {
            ContextHelper.ClearToken();
            var usersContext = ContextHelper.CreateUsersContext(false);
            Status.Notify(usersContext, null, "RequestToken");
            //Store the session token for use on other contexts.  Storing and re-using the session token
            //will avoid unnecessary authentication requests and improve response times.
            var sessionToken = usersContext.SessionToken;

            //Output
            Status.WriteLine(sessionToken);
            Status.EndNotify(usersContext);

            Status.Notify(usersContext, "User", "Get");
            //Fetch the user information using the current token applied to the header.
            usersContext.UserOperations.Get(int.Parse(Settings.Default.UserId));
            Status.EndNotify(usersContext);
        }

        //DO NOT CHANGE THE EXAMPLE NAME, the name is used verbatim in the Help documentation site.
        [Example("Instrument Search: Primary Example")]
        [EndToEndExample]
        public void Search()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();
            Status.Notify(extractionsContext, null, "InstrumentSearch");
            var results = extractionsContext.InstrumentSearch(
                IdentifierType.Ric,
                "US10YT=RRPS",
                new[] { 
                    InstrumentTypeGroup.CollatetizedMortgageObligations, InstrumentTypeGroup.Commodities, InstrumentTypeGroup.Equities, 
                    InstrumentTypeGroup.FuturesAndOptions, InstrumentTypeGroup.GovCorp, InstrumentTypeGroup.MortgageBackedSecurities,
                    InstrumentTypeGroup.Money, InstrumentTypeGroup.Municipals, InstrumentTypeGroup.Funds },
                IdentifierType.Ric, //Preferred identifier
                10); //Max results

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(extractionsContext);
        }

        //DO NOT CHANGE THE EXAMPLE NAME, the name is used verbatim in the Help documentation site.
        [Example("On Demand Extraction: Primary Example")]
        [EndToEndExample]
        public void OnDemandExtraction()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();
            Status.Notify(extractionsContext, null, "ExtractWithNotes");
            //Reduce the poll time for the purposes of the example
            extractionsContext.Preferences.WaitSeconds = 5;
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                IdentifierList = SubjectIdentifierList.CreateInstrumentIdentifierList(new[]
                    {
                       SubjectIdentifier.CreateInstrumentIdentifier("IBM.N", IdentifierType.Ric, null, null, null,  null)
                    }, null, false),
                ContentFieldNames = new[] { "Ask Price", "Asset Category", "Asset Category Description", "Asset ID", "Asset Status",
                "Asset Status Description", "Asset SubType", "Asset SubType Description", "Asset Type",
                "Asset Type Description", "Bid Price", "Block Trades", "CIN Code", "Common Code",
                "Currency Code", "Currency Code Description", "Currency Code Scaled", "Currency Code Scaled Description",
                "CUSIP", "Exchange Code", "Exchange Description", "Exercise Style", "Expiration Date", "File Code",
                "High Price", "Instrument ID", "Instrument ID Type", "ISIN", "Issuer OrgID", "Lot Size", "Low Price",
                "Market MIC", "MIC", "Mid Price", "Net Asset Value", "Offer Price",
                "Official Close Price", "Open Price", "Previous Close Price", "Put Call Indicator", "Quote ID",
                "RCP ID", "RIC", "Security Description", "SEDOL", "Settlement Date", "Settlement Price", "SICC",
                "Sicovam", "Strike Price", "Ticker", "Trade Date", "Trading Status", "Trading Symbol", "Turnover",
                "Underlying RIC", "Universal Ask Price", "Universal Bid Ask Date", "Universal Bid Price",
                "Usage Instrument SubType", "Usage Instrument Type", "Valoren", "Volume", "VWAP Price", "Wertpapier" }
            };

            //Extract
            var extraction = extractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extraction.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var field in extractedRows.First().DynamicProperties)
                    Status.WriteLine(field.Key + ": " + field.Value);
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extraction.Notes)
                Status.WriteLine(note);
            Status.EndNotify(extractionsContext);
        }

        //DO NOT CHANGE THE EXAMPLE NAME, the name is used verbatim in the Help documentation site.
        [Example("Scheduled Extraction: Primary Example")]
        [EndToEndExample]
        public void EndOfDay()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();
            Status.Notify(extractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = extractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EndOfDayPricing);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "ImmediateNonCustomized" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            extractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "ReportTemplates", "Create");
            //Define a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "ImmediateNonCustomized" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
                //NOTE: There are many other fields which can be use to customize the report template
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Ask Price", "Asset Category", "Asset Category Description", "Asset ID", "Asset Status",
                "Asset Status Description", "Asset SubType", "Asset SubType Description", "Asset Type",
                "Asset Type Description", "Bid Price", "Block Trades", "CIN Code", "Common Code",
                "Currency Code", "Currency Code Description", "Currency Code Scaled", "Currency Code Scaled Description",
                "CUSIP", "Exchange Code", "Exchange Description", "Exercise Style", "Expiration Date", "File Code",
                "High Price", "Instrument ID", "Instrument ID Type", "ISIN", "Issuer OrgID", "Lot Size", "Low Price",
                "Market MIC", "MIC", "Mid Price", "Net Asset Value", "Offer Price",
                "Official Close Price", "Open Price", "Previous Close Price", "Put Call Indicator", "Quote ID",
                "RCP ID", "RIC", "Security Description", "SEDOL", "Settlement Date", "Settlement Price", "SICC",
                "Sicovam", "Strike Price", "Ticker", "Trade Date", "Trading Status", "Trading Symbol", "Turnover",
                "Underlying RIC", "Universal Ask Price", "Universal Bid Ask Date", "Universal Bid Price",
                "Usage Instrument SubType", "Usage Instrument Type", "Valoren", "Volume", "VWAP Price", "Wertpapier");

            //Create the report template
            extractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "ImmediateNonCustomized" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true),
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            extractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(extractionsContext);

            try
            {
                Status.Notify(extractionsContext, "ReportExtraction", "GetLastExtractionForScheduleId");
                //Wait for the extraction to complete.  This involves polling GetLastExtractionForScheduleId until 
                //the returned ReportExtraction.Status is completed.
                var extraction = extractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);

                //Output some extraction information
                Status.WriteLine("status: {0} / detailedstatus: {1} / extraction date: {2} - {3} / files = {4}",
                    extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
                    extraction.Files.Count);
                Status.EndNotify(extractionsContext);

                Status.Notify(extractionsContext, "ReportExtraction", "Files");
                //Get the list of files for this extraction.  This includes the contents, notes files and RIC maintenance files.
                extractionsContext.LoadProperty(extraction, "Files");

                //Find the extract file.  Other ExtractedFileTypes are: Note, RicMaintenanceNote, Full, Partial, Other
                var extractFile = extraction.Files.FirstOrDefault(f => f.FileType == ExtractedFileType.Full);
                Status.EndNotify(extractionsContext);

                if (extractFile != null)
                {
                    Status.Notify(extractionsContext, "ReportExtraction", "GetReadStream");
                    //Output the contents of the extracted file
                    var streamResponse = extractionsContext.ExtractedFileOperations.GetReadStream(extractFile);
					string internalStreamOutput;
                    using (var reader = new StreamReader(streamResponse.Stream))
	                {
		                Status.WriteLine(internalStreamOutput = reader.ReadToEnd());
	                }
	                Status.EndNotify(extractionsContext, internalStreamOutput); // Extract the stream output and pass it as responseBody.
                }
            }
            catch (TimeoutException ex)
            {
                Status.WriteLine("Timed out: " + ex.Message);
            }

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        #region Cleanup Helper Method

        // This cleanup code serves as an example of how to do a get-all for the various entity types.
        // The cleanup button is useful if you abort debug sessions while exploring these samples.  
        // Cleanup is by name prefix.  It removes all lists, reports, schedules, and file content that matches the prefix.

        public void Cleanup(Schedule schedule, InstrumentList instrumentList, ReportTemplate reportTemplate)
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            if (schedule != null && schedule.ScheduleId != null)
            {
                var scheduleExtractions = extractionsContext.ReportExtractionOperations.GetByScheduleId(schedule.ScheduleId);
                foreach (var reportExtraction in scheduleExtractions)
                {
                    extractionsContext.LoadProperty(reportExtraction, "Files");
                    var fileIds = reportExtraction.Files
                                                  .Where(f => f.ContentsExists)
                                                  .Select(f => f.ExtractedFileId)
                                                  .ToList();
                    if (fileIds.Count > 0)
                        extractionsContext.ExtractedFileOperations.DeleteMultiple(fileIds);
                }
                extractionsContext.ScheduleOperations.Delete(schedule);
            }

            //Delete the report template
            if (reportTemplate != null && reportTemplate.ReportTemplateId != null)
                extractionsContext.ReportTemplateOperations.Delete(reportTemplate);

            //Delete the instrument list
            if (instrumentList != null && instrumentList.ListId != null)
                extractionsContext.InstrumentListOperations.Delete(instrumentList);

        }

        #endregion
    }
}
