﻿using Dss.Api.Examples.Internal;
using DataScope.Select.Api.Quota;

namespace Dss.Api.Examples.Quota
{
    [Example("Quota")]
    public class QuotaExamples
    {
        [EndToEndExample(typeof(QuotaContext))]
        [Example("Get Quota Information")]
        public void GetQuotaInformation()
        {
            //Create the context
            var quotaContext = ContextHelper.CreateQuotaContext();

            Status.Notify(quotaContext,null, "GetQuotaInformation", MethodType.Operation, Publish.Primary);
            //Get the quota information for the user. 
            var quotaInfo = quotaContext.GetQuotaInformation();
            
            //Output
            Status.WriteLine("User's Quota");
            foreach (var userQuota in quotaInfo)
            {
                Status.WriteLine(" Count : {0}", userQuota.Count);
                Status.WriteLine(" Limit : {0}", userQuota.Limit);
                Status.WriteLine(" Category : {0}", userQuota.Category);
            }
            Status.EndNotify(quotaContext);
        }

        [EndToEndExample(typeof(QuotaContext))]
        [Example("Get Authorized RIC Values By QuotaCategory")]
        public void GetAuthorizedRicList()
        {
            //Create the context
            var quotaContext = ContextHelper.CreateQuotaContext();

            Status.Notify(quotaContext, null, "GetAuthorizedRicList", MethodType.Operation, Publish.Primary);
            //Get the quota information for the user. 
            var quotaRecords = quotaContext.GetAuthorizedRicList("Cash");
            //Output
            Status.WriteLine("Authorized RIC values for '{0}' category", "Cash");
            foreach (var quotaRecord in quotaRecords)
            {
                Status.WriteLine(" Asset Class: {0}", quotaRecord.AssetClass);
                Status.WriteLine(" Authorization Date: {0}", quotaRecord.AuthorizationDate);
                Status.WriteLine(" Description : {0}", quotaRecord.AuthorizationDescription);
                Status.WriteLine(" Authorized User Id : {0}", quotaRecord.AuthorizationUserId);
                Status.WriteLine(" Authorized Value : {0}", quotaRecord.AuthorizedValue);
            }

            Status.EndNotify(quotaContext);
        }
    }
}
