﻿using System.Linq;
using Dss.Api.Examples.Internal;
using Dss.Api.Examples.Properties;
using DataScope.Select.Api.Users;

namespace Dss.Api.Examples.Users
{
    [Example("User Examples")]
    public class UserExamples
    {
        protected UsersContext UsersContext { get; set; }

        public UserExamples()
        {
            UsersContext = ContextHelper.CreateUsersContext();
        }

        [Example("Get All")]
        [EndToEndExample(typeof(UsersContext), typeof(User))]
        [EndToEndExample(typeof(UsersContext))]
        public void GetAll()
        {
            Status.Notify(UsersContext, "User", "GetAll", MethodType.Operation, Publish.Primary);
            var user = UsersContext.UserOperations.GetAll().First();

            //Output some information
            Status.WriteLine("username: " + user.UserName);
            Status.EndNotify(UsersContext);
        }

        [Example("Users: Get By Id")]
        [EndToEndExample(typeof(UsersContext), typeof(User))]
        public void GetById()
        {
            Status.Notify(UsersContext, "User", "Get", MethodType.Operation, Publish.Primary);
            var user = UsersContext.UserOperations.Get(int.Parse(Settings.Default.UserId));

            //Output some information
            Status.WriteLine("username: " + user.UserName);
            Status.EndNotify(UsersContext);
        }

        [Example("Lazy Load UserPreferences property")]
        [EndToEndExample(typeof(UsersContext))]
        [EndToEndExample(typeof(UsersContext), typeof(User))]
        [EndToEndExample(typeof(UsersContext), typeof(UserPreference))]
        public void LazyLoadUserPreferences()
        {
            Status.Notify(UsersContext, "User", "GetAll");
            //Fetch the user
            var user = UsersContext.UserOperations.GetAll().ToArray().First();
            Status.EndNotify(UsersContext);

            Status.Notify(UsersContext, "User", "Preferences", MethodType.Association, Publish.Primary);
            //Get the preferences by loading the User.UserPreferences.  Until this method is called, User.UserPreferences is null.
            UsersContext.LoadProperty(user, u => u.Preferences);

            //Output a few of the settings
            //NOTE: There are many other settings which have not yet been included in the beta.
            if (user.Preferences != null)
            {
                Status.WriteLine("date separator: " + user.Preferences.UiSettings.DateSeparator);
                Status.WriteLine("decimal separator: " + user.Preferences.UiSettings.DecimalSeparator); //There are more settings
                Status.WriteLine("preferred identifier: " + user.Preferences.ContentSettings.PreferredIdentifier); //There are more settings
            }
            Status.EndNotify(UsersContext);
        }

        [Example("Update User Preferences")]
        [EndToEndExample(typeof(UsersContext), typeof(UserPreference))]
        public void UpdateUserPreferences()
        {
            Status.Notify(UsersContext, "UserPreference", "GetAll", MethodType.Operation, Publish.Primary);
            //Fetch the user preferences
            var userPreferences = UsersContext.UserPreferenceOperations.GetAll().First();
            Status.EndNotify(UsersContext);

            Status.Notify(UsersContext, "UserPreference", "Update", MethodType.Operation, Publish.Primary);
            //Make changes
            userPreferences.ContentSettings.PartialEmbargoedReportsEnabled = false;
            UsersContext.UserPreferenceOperations.Update(userPreferences);
            Status.EndNotify(UsersContext);
        }

        [Example("Users: GetAll User claims")]
        [EndToEndExample(typeof(UsersContext), typeof(UserClaim))]
        public void UserClaim()
        {
            Status.Notify(UsersContext, "UserClaim", "GetAll", MethodType.Operation, Publish.Primary);
            var user = UsersContext.UserClaimOperations.GetAll().ToArray().First();

            //Output some information
            
            Status.WriteLine("User Id: " + user.UserId);
            Status.WriteLine("User ClaimId:", user.UserClaimId);
            Status.WriteLine("Type:", user.Type);
            Status.WriteLine("Subject:", user.Subject);
            Status.WriteLine("Value:", user.Value);
            
            Status.EndNotify(UsersContext);
        }
    }

}
